/*
 *  PyGiNaC Copyright (C) 2019 Vladimir V. Kisil
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <ginac/ginac.h>

#include <boost/python/class.hpp>
#include <boost/python/overloads.hpp>
#include <boost/python/def.hpp>
#include "pyginac.hpp"

namespace pyginac {

unsigned
register_new_function(std::string const & n, unsigned np = 1)
{
	return GiNaC::function::register_new(GiNaC::function_options(n, np));
}

//overloads for class function
	
BOOST_PYTHON_FUNCTION_OVERLOADS(register_new_function_overload, register_new_function, 1, 2)
	
// function class is wrapped in exprseq.cpp
void
wrap_function(void)
{
	using namespace boost::python;
	using GiNaC::ex;
	using GiNaC::basic;
	using GiNaC::exprseq;
	using GiNaC::exvector;
	using GiNaC::function;
	using GiNaC::fderivative;
	
	ex (function::* thiscontainer)(const exvector&) const = 
		&function::thiscontainer;
	
	class_<function, bases<exprseq, basic> >( "function", 
		"A symbolic function.  This class is used to implement builtin functions"
		" like sin(), cos(), etc. A user can define new functions as well.",
		init< unsigned, optional< const ex&, const ex&, const ex&, const ex&, const ex&,
											  const ex&, const ex&, const ex&, const ex&, const ex&,
											  const ex&, const ex&, const ex&> >())
		.def( "register_new", register_new_function, register_new_function_overload("Register new function with GiNaC"))
		.def( "get_name", &function::get_name)
		.def( "get_serial", &function::get_serial)
		.def( "thiscontainer", thiscontainer)
		.add_property( "name", &function::get_name)
		.add_property( "serial", &function::get_serial)
		;
		
	class_<fderivative, bases<function, exprseq, basic> >("fderivative",
		"The (abstract) derivative of a function.", no_init)
		;
}

} // !namespace ginac
